#ifndef NAGIOSAGENT_H
#define NAGIOSAGENT_H

#define QT_NO_DEBUG_OUTPUT

#include <QtDebug>
#include <QtGui>
#include <QKeySequence>
#include <QApplication>
#include <QThread>
#include <QSettings>
#include <QStringList>
#include <QHttp>
#include <QRegExp>
#include <QList>
#include <QSslError>
#include <QSslSocket>
#include "ui_nagiosagent.h"
#include "informationwindow.h"

#define MSG(s)	qDebug() << s

class NagiosAgent : public QWidget
{
    Q_OBJECT
public:
    NagiosAgent(QWidget *parent = 0);
    ~NagiosAgent();
private:
	enum httpCodes
	{
		httpOk = 200,
		httpMovedPerm = 301,
		httpMovedTemp = 302,
		httpNotFound = 404
	};
	enum tabNums
	{
		commonTab,
		filterHostsTab,
		filterServicesTab,
		statusTab,
		miscTab
	};
	enum menuNums
	{
		containerMenu,
		hostsMenu,
		srvcsMenu,
		mainMenu
	};

private Q_SLOTS:
    void	iconActivated(QSystemTrayIcon::ActivationReason reason);
    void	hostsSrvcsMenuHandler(QAction * action);
    void	addHost();
    void	delHost();
    void	addService();
    void	delService();
    void	checkAndSaveConfig();
    void	showAndReadConfig();
    void	processHttpFinished(int id, bool error);
    void	processSslErrors(const QList<QSslError> & sslErrorsList);
    void	processTimer();
    bool	isConfigOk();
    void	httpGetInfo();
    void	timerStart();
    void	closeAllPopups();
    void	showAbout();
    void	useHttpsClicked(bool use);
    void	trayMessageClicked();
    void	trayFlashed();
Q_SIGNALS:
	void closeAllInformWin();
private:
	static const int	defaultWidth = 507;
	static const int	defaultHeight = 370;
	static const int 	defaultRequestPeriod = 15; 	//seconds
	static const int	defaultMenuNumOnLMK = containerMenu;
	static const bool 	defaultAutoClose = false;
	static const bool	useUserDefExecDir = false;
	static const bool	useUserDefHostsRequest = false;
	static const bool	useUserDefSrvcsRequest = false;
	static const bool	useHTTPAuth = false;
	static const bool	defaultAllExceptSpecifedHosts = true;
	static const bool	defaultOnlyListedHosts = !defaultAllExceptSpecifedHosts;
	static const bool	defaultAllExceptSpecifedSrvcs = true;
	static const bool	defaultOnlyListedSrvcs = !defaultAllExceptSpecifedSrvcs;
	static const bool	defaultPopupsStayOnTop = true;
	static const bool	defaultUseHttps = false;
	static const bool	defaultDenyInvalidCert = true;		//Отвергаем SSL сертификаты с ошибками
	static const bool	defaultUseBasicRegExpConfig = true;	//Используем базовую конфигурацию фильтрации
	static const bool	defaultUseHostDownStat = true;
	static const bool	defaultUseHostUnreachStat = true;
	static const bool	defaultUseSrvCritStat = true;
	static const bool	defaultUseSrvWarnStat = true;
	static const bool	defaultUseSrvUnknStat = false;
	static const bool	defaultOpenUrlFromTray = true;
	static const bool	defaultDisableLocalization = false;
	static const bool	defaultFlashesInTray = true;
	static const int	maxHttpErrors = 6;			//Максимально допустимое число http ошибок подряд при выполнении запросов,
													//после превышения которого выдаёеся сообщения об ошибке
	static const int	defaultHostStatRxPos = 1;
	static const int	defaultHostLinkRxPos = 2;
	static const int	defaultHostNameRxPos = 3;
	static const int	defaultSrvStatRxPos = 1;
	static const int	defaultSrvLinkRxPos = 2;
	static const int	defaultSrvNameRxPos = 3;
	static const int	maxHttpConnectingTime = 30;	//Максимальное время в течение которого допустимо производить попытки
													//подключения к серверу без вывода сообщений об ошибках (секунды)
	static const int		flashingTimeout = 400;	//Таймаут мигания иконки в трее
	static const QString	defaultHttpServerPort;			//Default HTTP port
	static const QString	defaultHttpsServerPort;
	static const QString	defaultAutoCloseTime;		//minutes
	static const QString	defaultExecDir;
	static const QString	defaultHostRequest;
	static const QString	defaultSrvcsRequest;
	static const QString	defaultBasicHostsRegExp;
	static const QString	defaultBasicSrvcsRegExp;
	static const QString	defaultExpertHostsRegExp;
	static const QString	defaultExpertSrvcsRegExp;
	static const QString	trayToolTipHeader;
	static const QString	configDirName;
	static const QString	programmName;
	static const QString	hostUpSnd;
	static const QString	hostDownSnd;
	static const QString	programmTranslatorName;
	static const QString	commonTranslatorName;
	static const QString	normalTrayIcon;
	static const QString	problemTrayIcon;
	static const QString	hostUpIcon1;
	static const QString	hostUpIcon2;
	static const QString	hostDownIcon1;
	static const QString	hostDownIcon2;
	struct
	{
		QString serverAddress;
		QString	serverPort;
		int 	requestPeriod;
		int 	autoCloseTime;
		int		hostStatRxPos;
		int		hostLinkRxPos;
		int		hostNameRxPos;
		int		srvStatRxPos;
		int		srvLinkRxPos;
		int		srvNameRxPos;
		int		menuNumOnLMK;
		QString execDir;
		QString	hostsRequest;
		QString	servicesRequest;
		QString hostsRegExp;
		QString srvcsRegExp;
		bool 	autoClose;
		bool	useUserDefExecDir;
		bool 	useUserDefHostRequest;
		bool	useUserDefSrvcsRequest;
		bool	useHTTPAuth;
		bool	useUserDefHostsRegExp;
		bool	useUserDefSrvcsRegExp;
		bool	allExceptSpecifedHosts;
		bool	onlyListedHosts;
		bool	allExceptSpecifedSrvcs;
		bool	onlyListedSrvcs;
		bool	useTrayMessages;
		bool	disableSounds;
		bool	enableHosts;
		bool	enableSrvcs;
		bool	showHostsStateChanged;
		bool	showSrvcsStateChanged;
		bool	useHttps;
		bool	popupsAlwaysStayOnTop;
		bool	denyInvalidCert;
		bool	useBasicRegExpConfig;
		bool	openUrlFromTray;
		bool	disableLocalization;
		bool	flashesInTray;
		QString	httpLogin;
		QString httpPassword;
		QStringList hostsList;
		QStringList	srvcsList;
	}runningConfig;

	Ui::NagiosAgentClass ui;
    QTranslator commonTranslator;
    QTranslator programmTranslator;
    QDir		appDir;
	QSystemTrayIcon trayIcon;
	QMenu		trayHostsAndSrvcsMenu;
	QMenu 		trayMainMenu;
	QMenu 		trayHostsMenu;
	QMenu 		traySrvcsMenu;
	QHttp		* httpObj;
	QTimer 		timerRequests;
	QTimer		timerFlash;
	QPoint		informWinPt;
	QTcpSocket	* httpSocket;
	QSslSocket	* httpSslSocket;
	int			idGetHosts;
	int			idGetSrvcs;
	int			httpErrorsCount;
	int			httpConnectingTime;
	bool		ignoredHttpSignal;		//Переменная устанавливаемая в true после смены протокола (HTTP/HTTPS)
	bool		authReqErrorMsgVisible;
	bool		hasProblem;				//Устанавливается в true если в ходе выполнения запроса,
									//либо при начальной инициализации (чтении конфигурации) произошла ошибка
	bool		stateUp;
	const QString * currentIcon;
	int			flashingCounter;
	QString		sysTrayBufferStr;
	QString		rawHostsInfo;
	QString		rawSrvcsInfo;

	void	initAppResources();
	void	createAppDir();
	void	setCodecAndTranslators();
    void	closeEvent(QCloseEvent * event);
    void	createTrayIcon();
    void	connectSignals();
    void	createActions();
    void	readConfig();
    void	applyConfig();
    void	installTranslators(bool install);
    void	saveConfig();
    void	updateRunningConfig();
    void	processHostsInfo();
    void	processSrvcsInfo();
    void	updateInformWinPt();
    void	updateTrayToolTip(bool reset = false, QString reason = "");
    void	updateTrayMenus();
    void	reinitQHttp(bool httpsUsed, bool useHttps = false);
    void	updateBasicRegExp();
    void	showHostUpOrDownMessage(QString hostName, InformationTypes state,
				QString hostHttpLink, QString hostStatus);
    inline void trouble(QString reason);
    void	startFlashingTray(bool up);
};

#endif // NAGIOSAGENT_H
